## LME modelling for total rosette area data 

## Input: needs a data frame named "phenotype_table" with ar least the following columns:
# plant_id (factor): unique identifier for each plant
# experiment (factor): unique identifier for each experiment 
# genotype (factor): unique identifier for each genotype
# phenotype (factor): describes what kind of value "phenotype_value" contains for this row; "area" is necessary for the lme model (as well as "RGR" for the RGR plots, and "leaf_count" as well if this filter is to be used)
# phenotype_value (float): the value for the current row
# DASE (integer): the time (in days) for the current row 
# PAR (float): the light intensity measured for the row (set to same everywhere if unknown)


# Main options (see calls at the end of the script for typical usage)

# perform_lme_model(
#	phenotype_table						# The data to work on
#	reference_genotype = "Col-0", 				# Assume Col-0 if not set
#	genotype_filter = levels(phenotype_table$genotype), 	# Assume that we like to model all genotypes in the data
#	experiment_filter = levels(phenotype_table$experiment), # Assume we would like to model all experiments in the data
#	DASE_filter = c(-Inf,Inf), 				# Assume that we would like to cover the complete timeline
#	plant_leaf_count_min_filter = 12, 			# Plants should have reached the 12 leaf stage to be included
#	lme_model_rgr_slope_interaction = TRUE, 		# Model RGR slope interaction effects (= allow individual RGR slope for each genotype)? (recommended; therefore assume TRUE) 
#	lme_drop_non_significant_phenotypes = TRUE 		# Assume that we kick out non significant data
# )




# Needed libraries
library(nlme)
library(ggplot2)
library(multcomp)

# Logging function, replace if needed 
log_func <- print

# Plot functions
vp.layout <- function(x, y) viewport(layout.pos.row=x, layout.pos.col=y)
arrange <- function(..., nrow=NULL, ncol=NULL, as.table=FALSE) {
	dots <- list(...)
	n <- length(dots)
	if(is.null(nrow) & is.null(ncol)) { nrow = floor(n/2) ; ncol = ceiling(n/nrow)}
	if(is.null(nrow)) { nrow = ceiling(n/ncol)}
	if(is.null(ncol)) { ncol = ceiling(n/nrow)}
	
	grid.newpage()
	pushViewport(viewport(layout=grid.layout(nrow,ncol) ) )
	ii.p <- 1
	for(ii.row in seq(1, nrow)){
		ii.table.row <- ii.row 
		if(as.table) {ii.table.row <- nrow - ii.table.row + 1}
			for(ii.col in seq(1, ncol)){
			ii.table <- ii.p
			if(ii.p > n) break
			print(dots[[ii.table]], vp=vp.layout(ii.table.row, ii.col))
			ii.p <- ii.p + 1
		}
	}
}

# Outlier handling functions

find_and_plot_outlier_plants <- function(main_fit,area_data,max_points)	{
	# More than max_points being outside 2 SD? 
	std_residuals <- ((residuals(main_fit) - mean(residuals(main_fit)))/main_fit$sigma)
	outlier_plant_ids <- names(which(summary(as.factor(names(which(abs(std_residuals) > 1.96)))) > max_points))
	outlier_points <- (abs(std_residuals) > 1.96)
	fitted_points <- exp(fitted(main_fit))
	for (current_plant_id in outlier_plant_ids)	{
		if (current_plant_id != "(Other)")	{
			plot(main=paste("Outlier plant - ",current_plant_id,sep=""),xlab="DASE",ylab="Area",x=area_data[(area_data$plant_id == current_plant_id),c("DASE")],y=area_data[(area_data$plant_id == current_plant_id),c("phenotype_value")],log="y")
			points(x=area_data[(area_data$plant_id == current_plant_id) & outlier_points,c("DASE")],y=area_data[(area_data$plant_id == current_plant_id) & outlier_points,c("phenotype_value")],col="red",pch="x")
			points(x=area_data[(area_data$plant_id == current_plant_id),c("DASE")],y=fitted_points[(area_data$plant_id == current_plant_id)],col="blue",type="l")
			log_func(paste(" - Found outlier plant: ",current_plant_id,sep=""))
		}
	}
	return(outlier_plant_ids)
}

remove_outlier_plants_and_points <- function(max_points=7)	{
	# Retrieve first outliers
	outlier_plant_ids <- find_and_plot_outlier_plants(main_fit,area_data,max_points)

	# Iterate and remove until no outlier plants left:
	while((length(outlier_plant_ids) > 0) & (!((length(outlier_plant_ids) == 1) & (outlier_plant_ids[1] == "(Other)"))))	{
		# Remove them:
		area_data <<- area_data[!(area_data$plant_id %in% outlier_plant_ids),]
		rgr_data <<- rgr_data[!(rgr_data$plant_id %in% outlier_plant_ids),]
		
		# Re-fit the model
		main_fit <<- lme(main_formula,random=~1|plant_id,data=area_data)
		
		# Retrieve outliers
		outlier_plant_ids <- find_and_plot_outlier_plants(main_fit,area_data,max_points)
	}

	# Check for remaining outlier points; remove them
	std_residuals <- ((residuals(main_fit) - mean(residuals(main_fit)))/main_fit$sigma)
	outlier_lines <- (abs(std_residuals) > 1.96)

	# Remove them:
	area_data <<- area_data[!(outlier_lines),]
	log_func(paste(" - ",sum(outlier_lines), " outlier points were removed.",sep=""))
}

# Initialize "globals"
area_data <- NA
rgr_data <- NA
main_fit <- NA
main_formula <- NA
simple_formula <- NA

perform_lme_model <- function(
	phenotype_table,
	reference_genotype = "Col-0", 				# Assume Col-0 if not set
	genotype_filter = levels(phenotype_table$genotype), 	# Assume that we like to model all genotypes in the data
	experiment_filter = levels(phenotype_table$experiment), # Assume we would like to model all experiments in the data
	DASE_filter = c(-Inf,Inf), 				# Assume that we would like to cover the complete timeline
	plant_leaf_count_min_filter = 12, 			# Plants should have reached the 12 leaf stage to be included
	lme_model_rgr_slope_interaction = TRUE, 		# Model RGR slope interaction effects (= allow individual RGR slope for each genotype)? (recommended; therefore assume TRUE) 
	lme_drop_non_significant_phenotypes = TRUE 		# Assume that we kick out non significant data
)	{
	
	### Filter out unnecessary data

	# Find plants not reaching 12-leaf stage
	plant_ids_to_remove <- c()
	for (current_plant_id in unique(phenotype_table$plant_id))	{
		current_leaf_counts <- phenotype_table[(phenotype_table$phenotype == "leaf_count") & (phenotype_table$plant_id == current_plant_id),c("phenotype_value")]
		if (sum(current_leaf_counts > (plant_leaf_count_min_filter-1)) == 0)	{
			plant_ids_to_remove <- c(plant_ids_to_remove,current_plant_id)
		}
	}

	# Filter out such bad plants:
	lme_phenotype_table <- phenotype_table[!(phenotype_table$plant_id %in% plant_ids_to_remove),]

	# Filter genotype:
	lme_phenotype_table <- lme_phenotype_table[lme_phenotype_table$genotype %in% genotype_filter,]
	lme_phenotype_table$genotype <- as.factor(as.character(lme_phenotype_table$genotype))
	lme_phenotype_table$genotype <- relevel(lme_phenotype_table$genotype,ref=reference_genotype)

	# Filter experiment
	lme_phenotype_table <- lme_phenotype_table[(lme_phenotype_table$experiment %in% experiment_filter),]
	lme_phenotype_table$experiment <- as.factor(as.character(lme_phenotype_table$experiment))

	# Filter DASE
	lme_phenotype_table <- lme_phenotype_table[(lme_phenotype_table$DASE > (DASE_filter[1]-1)) & (lme_phenotype_table$DASE < (DASE_filter[2]+1)),]

	# Remove missing data
	lme_phenotype_table <- lme_phenotype_table[(!is.na(lme_phenotype_table$phenotype_value)) & (!is.na(lme_phenotype_table$DASE)) & (!is.na(lme_phenotype_table$plant_id)),]

	## Area data
	area_data <<- lme_phenotype_table[lme_phenotype_table$phenotype == "area",]

	## RGR data
	rgr_data <<- lme_phenotype_table[lme_phenotype_table$phenotype == "RGR",]

	### Formula selection

	## Set formulas
	log_func(paste("* Linear mixed effects model fit for the genotypes ",paste(levels(area_data$genotype),collapse=", ")," in the experiment(s) ",paste(levels(area_data$experiment),collapse=", "),sep=""))

	# Should we use a DASE^2 RGR correction slope for each genotype?
	if (lme_model_rgr_slope_interaction)	{
		log_func(" - We are modelling RGR slope interactions; set lme_model_rgr_slope_interaction to FALSE to avoid this.")
		# Check if we should add an experiment effect
		if (length(unique(area_data$experiment)) > 1)	{
			main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + PAR + genotype + experiment + (genotype*DASE) + (I(DASE^2)*genotype) + (PAR*DASE)
			simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + experiment + PAR
		} else	{
			main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + PAR + genotype + (genotype*DASE) + (I(DASE^2)*genotype) + (PAR*DASE)
			simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + PAR
		}
	} else	{
		# Check if we should add an experiment effect
		if (length(unique(area_data$experiment)) > 1)	{
			main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + PAR + genotype + experiment + (genotype*DASE) + (PAR*DASE)
			simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + experiment + PAR
		} else	{
			main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + PAR + genotype + (genotype*DASE) + (PAR*DASE)
			simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + PAR
		}
	}

	## Check whether PAR/distance_from_middle is significant; otherwise drop the term
	simple_fit <- lme(simple_formula,random=~1|plant_id,data=area_data)
	fit_summary <- summary(simple_fit)
	main_effects_table <- fit_summary$tTable
	PAR_p_val <- main_effects_table["PAR",c("p-value")]
	if (PAR_p_val > 0.05)	{
		log_func(paste(" - PAR effect on area is unsignificant, dropping the term (p = ",PAR_p_val,").",sep=""))
		if (lme_model_rgr_slope_interaction)	{
			# Check if we should add an experiment effect
			if (length(unique(area_data$experiment)) > 1)	{
				main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + experiment + (genotype*DASE) + (I(DASE^2)*genotype)
				simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + experiment
			} else	{
				main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + (genotype*DASE) + (I(DASE^2)*genotype)
				simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype
			}
		} else	{
			# Check if we should add an experiment effect
			if (length(unique(area_data$experiment)) > 1)	{
				main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + experiment + (genotype*DASE)
				simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + experiment
			} else	{
				main_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype + (genotype*DASE)
				simple_formula <<- log(phenotype_value) ~ DASE + I(DASE^2) + genotype 
			}
		}
	} else	{
		log_func(paste(" - PAR effect on area is significant, keeping the term (p = ",PAR_p_val,").",sep=""))
	}

	## Fit the model
	main_fit <<- lme(main_formula,random=~1|plant_id,data=area_data)
	
	### Outlier detection

	# Open a PDF for plotting failing plants
	pdf(paste("lme_model_outlier_plant_curves.pdf",sep=""),paper="a4r",width=0,height=0)
	par(mfrow=c(2,3))

	# Remove outlier plants and points:
	remove_outlier_plants_and_points()

	# Re-fit the model
	main_fit <<- lme(main_formula,random=~1|plant_id,data=area_data)
	fit_summary <- summary(main_fit)
	main_effects_table <- fit_summary$tTable

	## Run Tukey tests:
	log_func(" - Tukey range tests on simple LME model:")
	# Check if we should add an experiment effect
	fit_simple <- lme(simple_formula,random=~1|plant_id,data=area_data)
	glht_res <- summary(glht(fit_simple,linfct=mcp(genotype="Tukey")))
	print(glht_res)

	# Capture test names and p-values
	tukey_tests <- as.data.frame(cbind(names(glht_res$test$coefficients),glht_res$test$pvalues))
	colnames(tukey_tests) <- c("test","p_value")
	tukey_tests$p_value <- as.double(as.character(tukey_tests$p_value))

	## Check for non-significant lines; remove them (if configured to)
	genotypes_to_drop <- c()
	if (lme_drop_non_significant_phenotypes)	{
		for (current_genotype in levels(area_data$genotype))	{
			if (!(current_genotype %in% reference_genotype))	{
				# Check Tukey range results:
				rowname <- paste(current_genotype," - ",reference_genotype,sep="")
				if ((tukey_tests[(tukey_tests$test == rowname),]$p_value) > 0.05)	{
					log_func(paste(" - Genotype ",current_genotype," has no significant effect (Tukey range test); dropping.",sep=""))
					genotypes_to_drop <- c(genotypes_to_drop,current_genotype)
				}
			}
		}
	}
	if (length(genotypes_to_drop) > 0)	{
		# Remove them:
		area_data <<- area_data[!(area_data$genotype %in% genotypes_to_drop),]
		rgr_data <<- rgr_data[!(rgr_data$genotype %in% genotypes_to_drop),]
		log_func(paste(" - ",length(genotypes_to_drop), " genotypes were removed.",sep=""))
		
		# re-level factors
		area_data$genotype <<- as.factor(as.character(area_data$genotype))
		area_data$genotype <<- relevel(area_data$genotype,ref=reference_genotype)
		rgr_data$genotype <<- as.factor(as.character(rgr_data$genotype))
		rgr_data$genotype <<- relevel(rgr_data$genotype,ref=reference_genotype)
		
		# Re-fit the model
		main_fit <<- lme(main_formula,random=~1|plant_id,data=area_data)
		
		## Check for outliers again
		remove_outlier_plants_and_points()
	}

	# Close outlier plot PDF
	dev.off()

	### Final fit

	# Recalculate PAR to center it around the mean:
	area_data$PAR <<- area_data$PAR-mean(area_data$PAR)

	# Re-fit the model
	main_fit <<- lme(main_formula,random=~1|plant_id,data=area_data)
	fit_summary <- summary(main_fit)
	main_effects_table <- fit_summary$tTable

	## Print summaries
	log_func(" - Tukey range tests on simple LME model:")
	fit_simple <- lme(simple_formula,random=~1|plant_id,data=area_data)
	glht_res <- summary(glht(fit_simple,linfct=mcp(genotype="Tukey")))
	print(glht_res)

	log_func(" - Summary of LME model:")
	print(summary(main_fit))
	r_squared <- cor(fitted(main_fit), getResponse(main_fit))^2
	print(paste(" - R^2 estimation: ",r_squared,sep=""))
	
	log_func(" - ANOVA table for LME model:")
	print(anova(main_fit))

	# Get fixed coefs:
	intercept <- main_effects_table["(Intercept)",c("Value")]
	intercept_se <- main_effects_table["(Intercept)",c("Std.Error")]
	DASE_coef <- main_effects_table["DASE",c("Value")]
	DASE_coef_se <- main_effects_table["DASE",c("Std.Error")]
	DASE_sq_coef <- main_effects_table["I(DASE^2)",c("Value")]
	DASE_sq_coef_se <- main_effects_table["I(DASE^2)",c("Std.Error")]

	log_func(paste(" - Mean RGR: ",signif((100*DASE_coef),4)," % day^-1",sep=""))
	log_func(paste(" - Mean RGR slope: ",signif((2*100*DASE_sq_coef),4)," % day^-2",sep=""))

	if (!(PAR_p_val > 0.05))	{
		log_func(paste(" - PAR area effect: ",signif((100*(exp(main_effects_table["PAR",c("Value")])-1)),4)," % umol m2 s (PAR diff to mean)",sep=""))
		log_func(paste(" - Min PAR area effect (",signif(min(area_data$PAR),4)," umol m-2 s-1 diff): ",signif((100*(exp(min(area_data$PAR)*main_effects_table["PAR",c("Value")])-1)),4)," %",sep=""))
		log_func(paste(" - Max PAR area effect (",signif(max(area_data$PAR),4)," umol m-2 s-1 diff): ",signif((100*(exp(max(area_data$PAR)*main_effects_table["PAR",c("Value")])-1)),4)," %",sep=""))
		log_func(paste(" - PAR RGR effect: ",signif((100*main_effects_table["DASE:PAR",c("Value")]),4)," % day^-1 umol m2 s (PAR diff to mean)",sep=""))
		log_func(paste(" - Min PAR RGR effect (",signif(min(area_data$PAR),4)," umol m-2 s-1 diff): ",signif((min(area_data$PAR)*100*main_effects_table["DASE:PAR",c("Value")]),4)," % day^-1",sep=""))
		log_func(paste(" - Max PAR RGR effect (",signif(max(area_data$PAR),4)," umol m-2 s-1 diff): ",signif((max(area_data$PAR)*100*main_effects_table["DASE:PAR",c("Value")]),4)," % day^-1",sep=""))
	}

	# Collect effects in a table
	lme_table <- matrix(ncol=7,nrow=0)
	means_df <- matrix(ncol=1,nrow=0)
	rgr_means_df <- matrix(ncol=1,nrow=0)
	for (current_genotype in levels(area_data$genotype))	{
		if (!(current_genotype %in% reference_genotype))	{
			rowname <- paste("genotype",current_genotype,sep="")
			main_coef <- main_effects_table[rowname,c("Value")]
			main_p_val <- main_effects_table[rowname,c("p-value")]
			main_se <- main_effects_table[rowname,c("Std.Error")]
			if (main_p_val > 0.05)	{
				log_func(paste(" - ",current_genotype," has no significantly larger/smaller area.",sep=""))
			} else	{
				log_func(paste(" - ",current_genotype," has a significantly larger/smaller area (",signif((100*(exp(main_coef)-1)),4)," %).",sep=""))
			}
		} else	{
			rowname <- "(Intercept)"
			main_coef <- 0
			main_p_val <- 1
			main_se <- 0
			interaction_coef <- 0
			interaction_p_val <- 1
			interaction_se <- 0
		}
		# Get the days for this genotype
		start_day <- min(area_data[(area_data$genotype == current_genotype),c("DASE")])
		end_day <- max(area_data[(area_data$genotype == current_genotype),c("DASE")])
		days <- c(start_day:end_day)
		
		# Check if the model uses interaction between genotype and DASE
		interaction_rowname <- paste("DASE:",rowname,sep="")
		if (interaction_rowname %in% rownames(main_effects_table))	{
			# calculate coef:
			interaction_coef <- main_effects_table[interaction_rowname,c("Value")]
			interaction_p_val <- main_effects_table[interaction_rowname,c("p-value")]
			interaction_se <- main_effects_table[interaction_rowname,c("Std.Error")]
			if (interaction_p_val > 0.05)	{
				log_func(paste(" - ",current_genotype," has no significant RGR difference.",sep=""))
			} else	{
				log_func(paste(" - ",current_genotype," has a significant RGR difference (",signif((100*interaction_coef),4)," % day^-1).",sep=""))
			}
		} else	{
			interaction_coef <- 0
			interaction_p_val <- 1
			interaction_se <- 0
		}
		sq_interaction_rowname <- paste("I(DASE^2):",rowname,sep="")
		if (sq_interaction_rowname %in% rownames(main_effects_table))	{
			# calculate coef:
			sq_interaction_coef <- main_effects_table[sq_interaction_rowname,c("Value")]
			sq_interaction_p_val <- main_effects_table[sq_interaction_rowname,c("p-value")]
			sq_interaction_se <- main_effects_table[sq_interaction_rowname,c("Std.Error")]
			if (sq_interaction_p_val > 0.05)	{
				log_func(paste(" - ",current_genotype," has no significant RGR slope difference.",sep=""))
			} else	{
				log_func(paste(" - ",current_genotype," has a significant RGR slope difference.",sep=""))
			}
		} else	{
			sq_interaction_coef <- 0
			sq_interaction_p_val <- 1
			sq_interaction_se <- 0
		}
		
		# Put together summary table for this genotype
		if (!(current_genotype %in% reference_genotype))	{
			lme_table <- rbind(lme_table,cbind(current_genotype,((exp(main_coef)*100)-100),main_p_val,(interaction_coef*100),interaction_p_val,(2*sq_interaction_coef*100),sq_interaction_p_val))
		} else	{
			lme_table <- rbind(lme_table,cbind(current_genotype,((exp(main_coef)*100)-100),main_p_val,(DASE_coef*100),interaction_p_val,(2*DASE_sq_coef*100),sq_interaction_p_val))
		}
		
		# Calculate means, max and mins for each genotype
		mean_areas <- exp(intercept+main_coef+(days*interaction_coef)+(days*DASE_coef)+((days^2)*DASE_sq_coef)+((days^2)*sq_interaction_coef))
		low_error_areas <- exp(intercept+main_coef+(days*interaction_coef)+(days*DASE_coef)+((days^2)*DASE_sq_coef)+((days^2)*sq_interaction_coef)-(main_se+intercept_se+(DASE_coef_se)+(DASE_sq_coef_se)))
		high_error_areas <- exp(intercept+main_coef+(days*interaction_coef)+(days*DASE_coef)+((days^2)*DASE_sq_coef)+((days^2)*sq_interaction_coef)+(main_se+intercept_se+(DASE_coef_se)+(DASE_sq_coef_se)))
		
		# Calculate means, max and mins for RGR for each genotype
		mean_RGRs <- DASE_coef + 2*(DASE_sq_coef*days) + interaction_coef + 2*(sq_interaction_coef*days) 
		low_error_RGRs <- DASE_coef + 2*(DASE_sq_coef*days) + interaction_coef + 2*(sq_interaction_coef*days)
		
		# add for plotting
		rgr_means_df_appendix <- as.data.frame(cbind(rep(current_genotype,length(days)),days,mean_RGRs))
		colnames(rgr_means_df_appendix) <- c("genotype","DASE","mean_RGR")
		rgr_means_df_appendix$DASE <- as.integer(as.character(rgr_means_df_appendix$DASE))
		rgr_means_df_appendix$mean_RGR <- as.double(as.character(rgr_means_df_appendix$mean_RGR))
		if (dim(rgr_means_df)[1] == 0)	{
			rgr_means_df <- rgr_means_df_appendix
		} else	{
			rgr_means_df <- rbind(rgr_means_df,rgr_means_df_appendix)
		}
		
		means_df_appendix <- as.data.frame(cbind(rep(current_genotype,length(days)),days,mean_areas,low_error_areas,high_error_areas))
		colnames(means_df_appendix) <- c("genotype","DASE","mean_area","error_low","error_high")
		means_df_appendix$DASE <- as.integer(as.character(means_df_appendix$DASE))
		means_df_appendix$mean_area <- as.double(as.character(means_df_appendix$mean_area))
		means_df_appendix$error_low <- as.double(as.character(means_df_appendix$error_low))
		means_df_appendix$error_high <- as.double(as.character(means_df_appendix$error_high))
		if (dim(means_df)[1] == 0)	{
			means_df <- means_df_appendix
		} else	{
			means_df <- rbind(means_df,means_df_appendix)
		}
	}

	lme_table <- as.data.frame(lme_table)
	colnames(lme_table) <- c("genotype","area_change_percentage","area_change_p_value","overall_RGR_diff_percentage","overall_RGR_diff_p_value","RGR_slope_diff","RGR_slope_diff_p_value")
	lme_table$area_change_percentage <- as.double(as.character(lme_table$area_change_percentage))
	lme_table$area_change_p_value <- as.double(as.character(lme_table$area_change_p_value))
	lme_table$overall_RGR_diff_percentage <- as.double(as.character(lme_table$overall_RGR_diff_percentage))
	lme_table$overall_RGR_diff_p_value <- as.double(as.character(lme_table$overall_RGR_diff_p_value))
	lme_table$RGR_slope_diff <- as.double(as.character(lme_table$RGR_slope_diff))
	lme_table$RGR_slope_diff_p_value <- as.double(as.character(lme_table$RGR_slope_diff_p_value))

	# Save table with effects
	write.csv2(file=paste("lme_table.csv",sep=""),lme_table)

	### Model plots:

	# Create plot of model and SE and a scatter plot of original data:
	means_df$genotype = relevel(means_df$genotype,ref=reference_genotype)
	curr_plot <- ggplot(means_df,aes(x=DASE,y=mean_area,colour=genotype))+geom_line(size=1)
	curr_plot <- curr_plot + geom_errorbar(aes(ymin=error_low,ymax=error_high))
	curr_plot <- curr_plot + scale_y_log() + theme_bw(base_size = 18) + ylab(expression("Area, "*cm^2)) 
	curr_plot <- curr_plot + geom_point(data=area_data,aes(x=DASE,y=phenotype_value,group=plant_id,colour=genotype),size=1) #+ opts(axis.title.x = theme_text(vjust=-1,size=14)) 
	if (length(unique(means_df$genotype)) < 10)	{
		if (length(unique(means_df$genotype)) < 3)	{
			curr_plot <- curr_plot + scale_colour_manual(name="Genotype",values=c("#FDAE6B","#E6550D","#FEE6CE"))
		} else	{
			curr_plot <- curr_plot + scale_colour_brewer(name="Genotype",palette="OrRd")
		}
	}
	lme_plot <- curr_plot
	rm(curr_plot)

	# Create plot of modelled RGRs and a scatter plot of original RGR data:
	rgr_means_df$genotype = relevel(rgr_means_df$genotype,ref=reference_genotype)
	rgr_plot <- ggplot(rgr_means_df,aes(x=DASE,y=mean_RGR,colour=genotype))+geom_line(size=1)
	rgr_plot <- rgr_plot + scale_y_continuous(formatter="percent",limits=c(0,0.8)) + theme_bw(base_size = 18) + ylab(expression("RGR, % "*day^-1)) #+ opts(title="Mixed-effect model for RGRs (lines +/- SE) and real data (points)")
	rgr_plot <- rgr_plot + geom_point(data=rgr_data,aes(x=DASE,y=phenotype_value,group=plant_id,colour=genotype),size=1) #+ opts(axis.title.x = theme_text(vjust=-1,size=14)) 
	if (length(unique(rgr_means_df$genotype)) < 10)	{
		if (length(unique(rgr_means_df$genotype)) < 3)	{
			rgr_plot <- rgr_plot + scale_colour_manual(name="Genotype",values=c("#FDAE6B","#E6550D","#FEE6CE"))
		} else	{
			rgr_plot <- rgr_plot + scale_colour_brewer(name="Genotype",palette="OrRd")
		}
	}

	# Plot both plots on one page:
	pdf(file="lme_model_plots.pdf",paper="a4",width=0,height=0)
	arrange(lme_plot,rgr_plot,nrow=2,ncol=1)
	dev.off()

	# Hist of residuals
	curr_plot <- qplot(x=residuals(main_fit),geom="histogram",xlab="Residuals") + theme_bw(base_size = 18) 
	if (length(unique(means_df$genotype)) < 10)	{
		if (length(unique(means_df$genotype)) < 3)	{
			curr_plot <- curr_plot + scale_colour_manual(name="Genotype",values=c("#FDAE6B","#E6550D","#FEE6CE"))
		} else	{
			curr_plot <- curr_plot + scale_colour_brewer(name="Genotype",palette="OrRd")
		}
	}

	# Std residuals
	std_residuals <- ((residuals(main_fit) - mean(residuals(main_fit)))/main_fit$sigma)
	my_fitted <- fitted(main_fit)
	curr_plot2 <- qplot(y=std_residuals,x=my_fitted,colour=I("blue"),xlab="Fitted values",ylab="Standardized residuals") + theme_bw(base_size = 18) 
	curr_plot2 <- curr_plot2 + geom_hline(aes(yintercept=0),colour="red")

	# Std. resdiuals for each genotype
	plot_df <- area_data[,c("plant_id","genotype","DASE")]
	plot_df$fitted <- fitted(main_fit)
	plot_df$std_residuals <- ((residuals(main_fit) - mean(residuals(main_fit)))/main_fit$sigma)
	curr_plot3 <- ggplot(plot_df,aes(x=fitted,y=std_residuals,group=plant_id,colour=genotype))+geom_point()+ theme_bw(base_size = 18) 
	curr_plot3 <- curr_plot3 + geom_smooth(aes(group=genotype))
	curr_plot3 <- curr_plot3 + geom_hline(aes(yintercept=0),colour="#666666")
	curr_plot3 <- curr_plot3 + ylab("Standardized residuals") + xlab("Fitted values") 
	if (length(unique(plot_df$genotype)) < 10)	{
		if (length(unique(plot_df$genotype)) < 3)	{
			curr_plot3 <- curr_plot3 + scale_colour_manual(name="Genotype",values=c("#FDAE6B","#E6550D","#FEE6CE"))
		} else	{
			curr_plot3 <- curr_plot3 + scale_colour_brewer(name="Genotype",palette="OrRd")
		}
	}
	
	pdf("lme_model_residuals.pdf",paper="a4r",width=0,height=0)
	arrange(curr_plot,curr_plot2,curr_plot3,NA,nrow=2,ncol=2)
	dev.off()
	
}

############ Example calls:

# Load the supplied phenotype table 
load("phenotype_data_Col-0_sex4-3_grf9.Rdata")

## Model sex4-3 and Col-0
dir.create("sex4-3_model")
setwd("sex4-3_model")
# Model sex4-3 for all experiments (present everywhere)
perform_lme_model(phenotype_table, genotype_filter = c("Col-0","sex4-3"))
setwd("../")

## Model grf9 and Col-0
dir.create("grf9_model")
setwd("grf9_model")
# Model grf9 only for the experiment where it was included (to avoid an unbalanced design)
perform_lme_model(phenotype_table, genotype_filter = c("Col-0","grf9"),experiment_filter = unique(phenotype_table[phenotype_table$genotype == "grf9",c("experiment")]))
setwd("../")

